@@ -1074,7 +1074,18 @@ class SimCCMicrosoftAMD64(SimCC):
     ARCH = archinfo.ArchAMD64
     STACK_ALIGNMENT = 16
 
-class SimCCX86LinuxSyscall(SimCC):
+
+class SimCCSyscall(SimCC):
+    """
+    The base class of all syscall CCs.
+    """
+
+    @staticmethod
+    def syscall_num(state) -> int:
+        raise NotImplementedError()
+
+
+class SimCCX86LinuxSyscall(SimCCSyscall):
     ARG_REGS = ['ebx', 'ecx', 'edx', 'esi', 'edi', 'ebp']
     FP_ARG_REGS = []
     RETURN_VAL = SimRegArg('eax', 4)
@@ -1090,7 +1101,8 @@ class SimCCX86LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.eax
 
-class SimCCX86WindowsSyscall(SimCC):
+
+class SimCCX86WindowsSyscall(SimCCSyscall):
     # TODO: Make sure the information is correct
     ARG_REGS = [ ]
     FP_ARG_REGS = [ ]
@@ -1107,6 +1119,7 @@ class SimCCX86WindowsSyscall(SimCC):
     def syscall_num(state):
         return state.regs.eax
 
+
 class SimCCSystemVAMD64(SimCC):
     ARG_REGS = ['rdi', 'rsi', 'rdx', 'rcx', 'r8', 'r9']
     FP_ARG_REGS = ['xmm0', 'xmm1', 'xmm2', 'xmm3', 'xmm4', 'xmm5', 'xmm6', 'xmm7']
@@ -1147,7 +1160,8 @@ class SimCCSystemVAMD64(SimCC):
 
         return True
 
-class SimCCAMD64LinuxSyscall(SimCC):
+
+class SimCCAMD64LinuxSyscall(SimCCSyscall):
     ARG_REGS = ['rdi', 'rsi', 'rdx', 'r10', 'r8', 'r9']
     RETURN_VAL = SimRegArg('rax', 8)
     RETURN_ADDR = SimRegArg('ip_at_syscall', 8)
@@ -1162,7 +1176,8 @@ class SimCCAMD64LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.rax
 
-class SimCCAMD64WindowsSyscall(SimCC):
+
+class SimCCAMD64WindowsSyscall(SimCCSyscall):
     # TODO: Make sure the information is correct
     ARG_REGS = [ ]
     FP_ARG_REGS = [ ]
@@ -1179,6 +1194,7 @@ class SimCCAMD64WindowsSyscall(SimCC):
     def syscall_num(state):
         return state.regs.rax
 
+
 class SimCCARM(SimCC):
     ARG_REGS = [ 'r0', 'r1', 'r2', 'r3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1187,7 +1203,8 @@ class SimCCARM(SimCC):
     RETURN_VAL = SimRegArg('r0', 4)
     ARCH = archinfo.ArchARM
 
-class SimCCARMLinuxSyscall(SimCC):
+
+class SimCCARMLinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
     ARG_REGS = [ 'r0', 'r1', 'r2', 'r3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1204,6 +1221,7 @@ class SimCCARMLinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.r7
 
+
 class SimCCAArch64(SimCC):
     ARG_REGS = [ 'x0', 'x1', 'x2', 'x3', 'x4', 'x5', 'x6', 'x7' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1211,7 +1229,8 @@ class SimCCAArch64(SimCC):
     RETURN_VAL = SimRegArg('x0', 8)
     ARCH = archinfo.ArchAArch64
 
-class SimCCAArch64LinuxSyscall(SimCC):
+
+class SimCCAArch64LinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
     ARG_REGS = [ 'x0', 'x1', 'x2', 'x3', 'x4', 'x5', 'x6', 'x7' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1228,6 +1247,7 @@ class SimCCAArch64LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.x8
 
+
 class SimCCO32(SimCC):
     ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1237,7 +1257,8 @@ class SimCCO32(SimCC):
     RETURN_VAL = SimRegArg('v0', 4)
     ARCH = archinfo.ArchMIPS32
 
-class SimCCO32LinuxSyscall(SimCC):
+
+class SimCCO32LinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
     ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1286,9 +1307,9 @@ class SimCCO64(SimCC):      # TODO: add n32 and n64
     RETURN_VAL = SimRegArg('v0', 8)
     ARCH = archinfo.ArchMIPS64
 
-class SimCCO64LinuxSyscall(SimCC):
-    # TODO: Make sure all the information is correct
-    ARG_REGS = [ 'a0', 'a1', 'a2', 'a3' ]
+
+class SimCCO64LinuxSyscall(SimCCSyscall):
+    ARG_REGS = [ 'a0', 'a1', 'a2', 'a3', 'a4', 'a5', 'a6', 'a7' ]
     FP_ARG_REGS = []    # TODO: ???
     RETURN_VAL = SimRegArg('v0', 8)
     RETURN_ADDR = SimRegArg('ip_at_syscall', 8)
@@ -1316,7 +1337,8 @@ class SimCCPowerPC(SimCC):
     RETURN_VAL = SimRegArg('r3', 4)
     ARCH = archinfo.ArchPPC32
 
-class SimCCPowerPCLinuxSyscall(SimCC):
+
+class SimCCPowerPCLinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
     ARG_REGS = ['r3', 'r4', 'r5', 'r6', 'r7', 'r8', 'r9', 'r10']
     FP_ARG_REGS = [ ]
@@ -1333,6 +1355,30 @@ class SimCCPowerPCLinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.r0
 
+    @staticmethod
+    def linux_syscall_update_cr0(state, expr):
+        # special handling for Linux MIPS syscalls: $cr0_0 is used as an error flag (0 for success, 1 for error)
+        if state.project is not None and state.project.simos is not None and state.project.simos.name == 'Linux':
+            if state.solver.symbolic(expr):
+                state.regs.cr0_0 = 0
+            else:
+                expr_val = state.solver.eval(expr)
+                if state.arch.bits == 32 and expr_val > -515 & 0xffff_ffff:
+                    expr_val = -(-0x1_0000_0000 + expr_val)
+                    state.regs.cr0_0 = 1
+                elif state.arch.bits == 64 and expr_val > -515 & 0xffff_ffff_ffff_ffff:
+                    expr_val = -(-0x1_0000_0000_0000_0000 + expr_val)
+                    state.regs.cr0_0 = 1
+                else:
+                    state.regs.cr0_0 = 0
+                expr = state.solver.BVV(expr_val, state.arch.bits)
+        return expr
+
+    def set_return_val(self, state, val, is_fp=None, size=None, stack_base=None):
+        new_val = SimCCPowerPCLinuxSyscall.linux_syscall_update_cr0(state, val)
+        super().set_return_val(state, new_val, is_fp=is_fp, size=size, stack_base=stack_base)
+
+
 class SimCCPowerPC64(SimCC):
     ARG_REGS = [ 'r3', 'r4', 'r5', 'r6', 'r7', 'r8', 'r9', 'r10' ]
     FP_ARG_REGS = []    # TODO: ???
@@ -1341,9 +1387,10 @@ class SimCCPowerPC64(SimCC):
     RETURN_VAL = SimRegArg('r3', 8)
     ARCH = archinfo.ArchPPC64
 
-class SimCCPowerPC64LinuxSyscall(SimCC):
+
+class SimCCPowerPC64LinuxSyscall(SimCCSyscall):
     # TODO: Make sure all the information is correct
-    ARG_REGS = [ ]
+    ARG_REGS = ['r3', 'r4', 'r5', 'r6', 'r7', 'r8', 'r9', 'r10']
     FP_ARG_REGS = [ ]
     RETURN_VAL = SimRegArg('r3', 8)
     RETURN_ADDR = SimRegArg('ip_at_syscall', 8)
@@ -1358,10 +1405,16 @@ class SimCCPowerPC64LinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.r0
 
+    def set_return_val(self, state, val, is_fp=None, size=None, stack_base=None):
+        new_val = SimCCPowerPCLinuxSyscall.linux_syscall_update_cr0(state, val)
+        super().set_return_val(state, new_val, is_fp=is_fp, size=size, stack_base=stack_base)
+
+
 class SimCCSoot(SimCC):
     ARCH = archinfo.ArchSoot
     ARG_REGS = []
 
+
 class SimCCUnknown(SimCC):
     """
     Represent an unknown calling convention.
@@ -1374,6 +1427,7 @@ class SimCCUnknown(SimCC):
     def __repr__(self):
         return "<SimCCUnknown - %s %s sp_delta=%d>" % (self.arch.name, self.args, self.sp_delta)
 
+
 class SimCCS390X(SimCC):
     ARG_REGS = ['r2', 'r3', 'r4', 'r5', 'r6']
     FP_ARG_REGS = ['f0', 'f2', 'f4', 'f6']
@@ -1382,7 +1436,8 @@ class SimCCS390X(SimCC):
     RETURN_VAL = SimRegArg('r2', 8)
     ARCH = archinfo.ArchS390X
 
-class SimCCS390XLinuxSyscall(SimCC):
+
+class SimCCS390XLinuxSyscall(SimCCSyscall):
     ARG_REGS = ['r2', 'r3', 'r4', 'r5', 'r6', 'r7']
     FP_ARG_REGS = []
     RETURN_VAL = SimRegArg('r2', 8)
@@ -1398,6 +1453,7 @@ class SimCCS390XLinuxSyscall(SimCC):
     def syscall_num(state):
         return state.regs.r1
 
+
 CC = {
     'AMD64': [
         SimCCSystemVAMD64,
@@ -1456,7 +1512,8 @@ DEFAULT_CC: Dict[str,Type[SimCC]] = {
 def register_default_cc(arch, cc):
     DEFAULT_CC[arch] = cc
 
-SYSCALL_CC = {
+
+SYSCALL_CC: Dict[str,Dict[str,Type[SimCCSyscall]]] = {
     'X86': {
         'default': SimCCX86LinuxSyscall,
         'Linux': SimCCX86LinuxSyscall,
