@@ -116,6 +116,7 @@ class SimSuccessors:
         state.scratch.source = source if source is not None else self.addr
         state.scratch.exit_stmt_idx = exit_stmt_idx
         state.scratch.exit_ins_addr = exit_ins_addr
+        state.history.jump_source = state.scratch.exit_ins_addr
 
         self._preprocess_successor(state, add_guard=add_guard)
 
@@ -521,5 +522,5 @@ from ..errors import SimSolverModeError, AngrUnsupportedSyscallError, AngrSyscal
 from ..calling_conventions import SYSCALL_CC
 from ..state_plugins.sim_action_object import _raw_ast
 from ..state_plugins.callstack import CallStack
-from ..storage.memory import DUMMY_SYMBOLIC_READ_VALUE
+from ..storage import DUMMY_SYMBOLIC_READ_VALUE
 from .. import sim_options as o
