@@ -21,7 +21,7 @@ class DataSet:
     Addition and subtraction are performed on the cartesian product of the operands. Duplicate results are removed.
     Data must always include a set.
 
-    :ivar set data:    The set of data to represent.
+    :ivar set[Union[int,str,Undefined]] data: The set of data to represent.
     :ivar Union[int,UnknownSize] bits: The size of an element of the set, in number of bits its representation takes.
     """
     maximum_size = 5
@@ -57,8 +57,17 @@ class DataSet:
         if self._bits <= bits:
             return DataSet(self.data, bits)
 
-        mask = (1 << bits) - 1
-        data = { d & mask if isinstance(d, int) else d for d in self.data }
+        def _truncate(datum):
+            if isinstance(datum, int):
+                mask = (1 << bits) - 1
+                return datum & mask
+            elif isinstance(datum, str):
+                number_of_bytes = bits // 8
+                return datum[:number_of_bytes]
+            else:
+                return datum
+
+        data = { _truncate(d) for d in self.data }
         return DataSet(data, bits)
 
     def update(self, data):
@@ -193,16 +202,28 @@ def _stringify_datum(datum):
     return datum
 
 
+def size_of_datasets(datasets: List[DataSet]) -> DataSet:
+    """
+    If the size of all datasets are equal, then the result will be of that size; Otherwise, it will be of size UNKNOWN_SIZE.
+
+    :param datasets: The datasets to get the size from.
+    :return: The common size of the given datasets.
+    """
+    return (
+        datasets[0]._bits if all(map(lambda d: d._bits == datasets[0]._bits, datasets))
+        else UNKNOWN_SIZE
+    )
+
+
+
 def dataset_from_datasets(datasets: List[DataSet]) -> DataSet:
     """
     Instantiate a <DataSet> by "merging" several <DataSet>s.
-    If the size of all datasets are equal, then the result will be of that size; Otherwise, it will be of size UNKNOWN_SIZE.
 
     :param datasets: The datasets to merge into one.
     :return: A new <DataSet>, containing the data from all given datasets.
     """
-    size = (datasets[0]._bits if all(map(lambda d: d._bits == datasets[0]._bits, datasets))
-            else UNKNOWN_SIZE)
+    size = size_of_datasets(datasets)
 
     result = DataSet(set(), size)
     for ds in datasets:
