@@ -85,7 +85,7 @@ class SimFilesystem(SimStatePlugin): # pretends links don't exist
             try:
                 subdeck = [o._mountpoints[fname] for o in others]
             except KeyError:
-                raise SimMergeError("Can't merge filesystems with disparate file sets")
+                raise SimMergeError("Can't merge filesystems with disparate file sets") # pylint: disable=raise-missing-from
 
             if common_ancestor is not None and fname in common_ancestor._mountpoints:
                 common_mp = common_ancestor._mountpoints[fname]
@@ -268,6 +268,16 @@ class SimMount(SimStatePlugin):
         """
         raise NotImplementedError
 
+    def lookup(self, sim_file):
+        """
+        Look up the path of a SimFile in the mountpoint
+
+        :param sim_file:        A SimFile object needs to be looked up
+        :return:                A string representing the path of the file in the mountpoint
+                                Or None if the SimFile does not exist in the mountpoint
+        """
+        raise NotImplementedError
+
 class SimConcreteFilesystem(SimMount):
     """
     Abstract SimMount allowing the user to import files from some external source into the guest
@@ -310,6 +320,12 @@ class SimConcreteFilesystem(SimMount):
         self.deleted_list.add(path)
         return self.cache.pop(path, None) is not None
 
+    def lookup(self, sim_file):
+        for key, val in self.cache.items():
+            if sim_file == val:
+                return key
+        return None
+
     @SimStatePlugin.memo
     def copy(self, memo):
         x = type(self)(pathsep=self.pathsep)
@@ -362,6 +378,7 @@ class SimConcreteFilesystem(SimMount):
         """
         return self.pathsep + self.pathsep.join(keys)
 
+
 class SimHostFilesystem(SimConcreteFilesystem):
     """
     Simulated mount that makes some piece from the host filesystem available to the guest.
